﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;

namespace FieldProxyBinding
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = new MainVM();
        }
    }
       
    
    public class MainVM: INotifyPropertyChanged
    {

        private string nameHeader = "Name";
        public string NameHeader
        {
            get { return nameHeader; }
            set { nameHeader = value; OnPropertyChanged("NameHeader"); }
        }
        
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    #region Field Binding Proxy

    public class FieldBindingProxy : DependencyObject
    {
        #region Private Fields
        private Field _owner;
        private DataPresenterBase _grid;
        #endregion

        #region Initializing Methods
        public void AttachOwner(Field field)
        {
            if (_owner != null)
            {
                DetachOwner(_owner);
            }

            _owner = field;
            Dispatcher.BeginInvoke(new Action(() =>
            {
                _grid = _owner.Owner.DataPresenter;
                InitializeAndSetBinding();
            }));
        }

        public void DetachOwner(Field field)
        {
            if (_owner != field)
            {
                return;
            }
            BindingOperations.ClearBinding(_owner, typeof(Field).GetField(FieldPropertyPath + "Property").GetValue(null) as DependencyProperty);

            _owner = null;
            _grid = null;
        }
        #endregion

        #region Properties

        #region BindingSource
        public BindingSource BindingSource
        {
            get { return (BindingSource)GetValue(BindingSourceProperty); }
            set { SetValue(BindingSourceProperty, value); }
        }

        public static readonly DependencyProperty BindingSourceProperty =
            DependencyProperty.Register(
                "BindingSource",
                typeof(BindingSource),
                typeof(FieldBindingProxy),
                new PropertyMetadata(BindingSource.DataContext,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));
        #endregion

        #region PropertyPath

        public string PropertyPath
        {
            get { return (string)GetValue(PropertyPathProperty); }
            set { SetValue(PropertyPathProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PropertyPath.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty PropertyPathProperty =
            DependencyProperty.Register(
                "PropertyPath",
                typeof(string),
                typeof(FieldBindingProxy),
                new PropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));


        #endregion

        #region Converter

        public IValueConverter Converter
        {
            get { return (IValueConverter)GetValue(ConverterProperty); }
            set { SetValue(ConverterProperty, value); }
        }

        public static readonly DependencyProperty ConverterProperty =
            DependencyProperty.Register(
                "Converter",
                typeof(IValueConverter),
                typeof(FieldBindingProxy),
                new PropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));


        #endregion

        #region ConverterParameter


        public object ConverterParameter
        {
            get { return (object)GetValue(ConverterParameterProperty); }
            set { SetValue(ConverterParameterProperty, value); }
        }

        public static readonly DependencyProperty ConverterParameterProperty =
            DependencyProperty.Register(
                "ConverterParameter",
                typeof(object),
                typeof(FieldBindingProxy),
                new UIPropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));


        #endregion

        #region FieldPropertyPatgh

        public string FieldPropertyPath
        {
            get { return (string)GetValue(FieldPropertyPathProperty); }
            set { SetValue(FieldPropertyPathProperty, value); }
        }

        public static readonly DependencyProperty FieldPropertyPathProperty =
            DependencyProperty.Register(
                "FieldPropertyPath",
                typeof(string),
                typeof(FieldBindingProxy),
                new UIPropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));

        #endregion

        #endregion

        #region Helper Methods

        private void InitializeAndSetBinding()
        {
            if (_owner == null || _grid == null)
                return;

            if (string.IsNullOrEmpty(PropertyPath))
                return;

            Binding binding = new Binding(PropertyPath);

            switch (BindingSource)
            {
                case BindingSource.DataContext:
                    binding.Source = _grid.DataContext;
                    break;
                case BindingSource.XamDataGrid:
                    binding.Source = _grid;
                    break;
                default:
                    break;
            }

            if (Converter != null)
                binding.Converter = Converter;

            if (ConverterParameter != null)
                binding.ConverterParameter = ConverterParameter;


            BindingOperations.SetBinding(_owner, typeof(Field).GetField(FieldPropertyPath + "Property").GetValue(null) as DependencyProperty, binding);
        }

        #endregion

        #region Attached Property

        public static readonly DependencyProperty FieldVisibilityBindingProperty =
            DependencyProperty.RegisterAttached(
                "FieldVisibilityBinding",
                typeof(FieldBindingProxy),
                typeof(FieldBindingProxy),
                new PropertyMetadata(null,
                    (o, e) =>
                    {
                        FieldVisibilityBindingChanged(o as Field,
                            e.OldValue as FieldBindingProxy,
                            e.NewValue as FieldBindingProxy);
                    }));

        public static FieldBindingProxy GetFieldVisibilityBinding(DependencyObject target)
        {
            return target.GetValue(FieldVisibilityBindingProperty) as FieldBindingProxy;
        }

        public static void SetFieldVisibilityBinding(DependencyObject target, FieldBindingProxy value)
        {
            target.SetValue(FieldVisibilityBindingProperty, value);
        }

        public static void FieldVisibilityBindingChanged(
            Field field,
            FieldBindingProxy oldVal,
            FieldBindingProxy newVal)
        {
            if (field == null)
            {
                return;
            }
            if (oldVal != null)
            {
                oldVal.DetachOwner(field);
            }
            if (newVal != null)
            {
                newVal.AttachOwner(field);
            }
        }

        #endregion

    }

    public enum BindingSource
    {
        /// <summary>
        /// Use to set the source of the Visibility Binding to the DataContext of the XamDataGrid
        /// </summary>
        DataContext,
        /// <summary>
        /// Use to set the source of the Visibility Binding to the XamDataGrid itself, when using RelativeSource 
        /// </summary>
        XamDataGrid
    }

    #endregion
				
}
